import { TsConfigJson } from "get-tsconfig";
import { IsolatedDeclarationsOptions } from "rolldown/experimental";
import { Plugin } from "rolldown";

//#region src/fake-js.d.ts
declare function createFakeJsPlugin({
  dtsInput,
  sourcemap
}: Pick<OptionsResolved, "dtsInput" | "sourcemap">): Plugin;
//#endregion
//#region src/generate.d.ts
declare function createGeneratePlugin({
  tsconfigRaw,
  tsconfigDir,
  isolatedDeclarations,
  emitDtsOnly,
  vue,
  parallel,
  eager
}: Pick<OptionsResolved, "tsconfigRaw" | "tsconfigDir" | "isolatedDeclarations" | "emitDtsOnly" | "vue" | "parallel" | "eager">): Plugin;
//#endregion
//#region src/utils/filename.d.ts
declare const RE_JS: RegExp;
declare const RE_TS: RegExp;
declare const RE_DTS: RegExp;
declare const RE_DTS_MAP: RegExp;
declare const RE_NODE_MODULES: RegExp;
declare const RE_CSS: RegExp;
declare const RE_VUE: RegExp;
//#endregion
//#region src/index.d.ts
interface Options {
  /**
  * The directory in which the plugin will search for the `tsconfig.json` file.
  */
  cwd?: string;
  /**
  * Set to `true` if your entry files are `.d.ts` files instead of `.ts` files.
  *
  * When enabled, the plugin will skip generating a `.d.ts` file for the entry point.
  */
  dtsInput?: boolean;
  /**
  * If `true`, the plugin will emit only `.d.ts` files and remove all other output chunks.
  *
  * This is especially useful when generating `.d.ts` files for the CommonJS format as part of a separate build step.
  */
  emitDtsOnly?: boolean;
  /**
  * The path to the `tsconfig.json` file.
  *
  * If set to `false`, the plugin will ignore any `tsconfig.json` file.
  * You can still specify `compilerOptions` directly in the options.
  *
  * @default 'tsconfig.json'
  */
  tsconfig?: string | boolean;
  /**
  * Pass a raw `tsconfig.json` object directly to the plugin.
  *
  * @see https://www.typescriptlang.org/tsconfig
  */
  tsconfigRaw?: Omit<TsConfigJson, "compilerOptions">;
  /**
  * Override the `compilerOptions` specified in `tsconfig.json`.
  *
  * @see https://www.typescriptlang.org/tsconfig/#compilerOptions
  */
  compilerOptions?: TsConfigJson.CompilerOptions;
  /**
  * If `true`, the plugin will generate `.d.ts` files using Oxc,
  * which is significantly faster than the TypeScript compiler.
  *
  * This option is automatically enabled when `isolatedDeclarations` in `compilerOptions` is set to `true`.
  */
  isolatedDeclarations?: boolean | Omit<IsolatedDeclarationsOptions, "sourcemap">;
  /**
  * If `true`, the plugin will generate declaration maps (`.d.ts.map`) for `.d.ts` files.
  */
  sourcemap?: boolean;
  /**
  * Resolve external types used in `.d.ts` files from `node_modules`.
  */
  resolve?: boolean | (string | RegExp)[];
  /**
  * If `true`, the plugin will generate `.d.ts` files using `vue-tsc`.
  */
  vue?: boolean;
  /**
  * If `true`, the plugin will launch a separate process for `tsc` or `vue-tsc`.
  * This enables processing multiple projects in parallel.
  */
  parallel?: boolean;
  /**
  * If `true`, the plugin will prepare all files listed in `tsconfig.json` for `tsc` or `vue-tsc`.
  *
  * This is especially useful when you have a single `tsconfig.json` for multiple projects in a monorepo.
  */
  eager?: boolean;
}
type Overwrite<T, U> = Pick<T, Exclude<keyof T, keyof U>> & U;
type OptionsResolved = Overwrite<Required<Omit<Options, "compilerOptions">>, {
  tsconfig: string | undefined;
  isolatedDeclarations: IsolatedDeclarationsOptions | false;
  tsconfigRaw: TsConfigJson;
  tsconfigDir: string;
}>;
declare function dts(options?: Options): Plugin[];
declare function resolveOptions({
  cwd,
  tsconfig,
  compilerOptions,
  tsconfigRaw: overriddenTsconfigRaw,
  isolatedDeclarations,
  sourcemap,
  dtsInput,
  emitDtsOnly,
  resolve,
  vue,
  parallel,
  eager
}: Options): OptionsResolved;
//#endregion
export { Options, OptionsResolved, RE_CSS, RE_DTS, RE_DTS_MAP, RE_JS, RE_NODE_MODULES, RE_TS, RE_VUE, createFakeJsPlugin, createGeneratePlugin, dts, resolveOptions };