import { createRequire } from "node:module";
import Debug from "debug";
import ts from "typescript";

//#region src/utils/vue.ts
const debug$1 = Debug("rolldown-plugin-dts:vue");
let createVueProgram;
const require = createRequire(import.meta.url);
function loadVueLanguageTools() {
	try {
		const vueTscPath = require.resolve("vue-tsc");
		const { proxyCreateProgram } = require(require.resolve("@volar/typescript", { paths: [vueTscPath] }));
		const vue = require(require.resolve("@vue/language-core", { paths: [vueTscPath] }));
		return {
			proxyCreateProgram,
			vue
		};
	} catch (error) {
		debug$1("vue language tools not found", error);
		throw new Error("Failed to load vue language tools. Please manually install vue-tsc.");
	}
}
function createVueProgramFactory(ts$1) {
	if (createVueProgram) return createVueProgram;
	debug$1("loading vue language tools");
	const { proxyCreateProgram, vue } = loadVueLanguageTools();
	return createVueProgram = proxyCreateProgram(ts$1, ts$1.createProgram, (ts$2, options) => {
		const { configFilePath } = options.options;
		const vueOptions = typeof configFilePath === "string" ? vue.createParsedCommandLine(ts$2, ts$2.sys, configFilePath.replaceAll("\\", "/")).vueOptions : vue.getDefaultCompilerOptions();
		const vueLanguagePlugin = vue.createVueLanguagePlugin(ts$2, options.options, vueOptions, (id) => id);
		return { languagePlugins: [vueLanguagePlugin] };
	});
}

//#endregion
//#region src/utils/tsc.ts
const debug = Debug("rolldown-plugin-dts:tsc");
debug(`loaded typescript: ${ts.version}`);
const programs = [];
const formatHost = {
	getCurrentDirectory: () => ts.sys.getCurrentDirectory(),
	getNewLine: () => ts.sys.newLine,
	getCanonicalFileName: ts.sys.useCaseSensitiveFileNames ? (f) => f : (f) => f.toLowerCase()
};
const defaultCompilerOptions = {
	declaration: true,
	noEmit: false,
	emitDeclarationOnly: true,
	noEmitOnError: true,
	checkJs: false,
	declarationMap: false,
	skipLibCheck: true,
	target: 99,
	resolveJsonModule: true,
	moduleResolution: ts.ModuleResolutionKind.Bundler
};
function createOrGetTsModule(options) {
	const { id, entries } = options;
	const program = programs.find((program$1) => {
		const roots = program$1.getRootFileNames();
		if (entries) return entries.every((entry) => roots.includes(entry));
		return roots.includes(id);
	});
	if (program) {
		const sourceFile = program.getSourceFile(id);
		if (sourceFile) return {
			program,
			file: sourceFile
		};
	}
	debug(`create program for module: ${id}`);
	const module = createTsProgram(options);
	debug(`created program for module: ${id}`);
	programs.push(module.program);
	return module;
}
function createTsProgram({ entries, id, tsconfigRaw, tsconfigDir, vue }) {
	const parsedCmd = ts.parseJsonConfigFileContent(tsconfigRaw, ts.sys, tsconfigDir);
	const compilerOptions = {
		...defaultCompilerOptions,
		...parsedCmd.options
	};
	const rootNames = [...new Set([id, ...entries || parsedCmd.fileNames].map((f) => ts.sys.resolvePath(f)))];
	const host = ts.createCompilerHost(compilerOptions, true);
	const createProgram = vue ? createVueProgramFactory(ts) : ts.createProgram;
	const program = createProgram({
		rootNames,
		options: compilerOptions,
		host,
		projectReferences: parsedCmd.projectReferences
	});
	const sourceFile = program.getSourceFile(id);
	if (!sourceFile) throw new Error(`Source file not found: ${id}`);
	return {
		program,
		file: sourceFile
	};
}
function tscEmit(tscOptions) {
	const module = createOrGetTsModule(tscOptions);
	const { program, file } = module;
	let dtsCode;
	let map;
	const { emitSkipped, diagnostics } = program.emit(file, (fileName, code) => {
		if (fileName.endsWith(".map")) {
			debug(`emit dts sourcemap: ${fileName}`);
			map = JSON.parse(code);
		} else {
			debug(`emit dts: ${fileName}`);
			dtsCode = code;
		}
	}, void 0, true, void 0, true);
	if (emitSkipped && diagnostics.length) return { error: ts.formatDiagnostics(diagnostics, formatHost) };
	return {
		code: dtsCode,
		map
	};
}

//#endregion
export { tscEmit };